/* groundcrew for connect - view handler */
var fs = require('fs');
var _renderer = {};

exports.new = function(renderer,extension,caching)
{
    // This needs to clean up our object store
    _renderer = {};

    _renderer.engine = {};
    // default for caching is off
    if (caching == undefined)
    {
        if (_renderer.caching == undefined) { cache(false); }
    } else 
    {
        cache(caching);
    }

    if (renderer) {
        registerEngine(renderer,extension);
    }

    return true;
}

//Lets us register our renderer. This can be called multiple times.
registerEngine = exports.registerEngine = function(renderer,extension)
{
    if (!renderer) throw new Error("renderer is required");

    // set up our renderer - the default is if we do not provide an extension.
    // Any template (file) that does not have an extension mapped to a specific
    // renderer
    if (!extension) { extension = '_default'; }
    extension = extension.toLowerCase();
    if (!_renderer.engine) _renderer.engine = {};
    _renderer.engine[extension] = renderer;

    // Make sure templates is initialized
    _renderer.engine[extension].templates = {};

    return true;
}

unregisterEngine = exports.unregisterEngine = function(ext)
{
    if (_renderer.engine[ext] == undefined) 
       throw new ReferenceError("Can't unregister " + ext + " because it is not registered");

    if (Object.keys(_renderer.engine).length <= 1)
        throw new SyntaxError("Can't unregister default engine");

    delete _renderer.engine[ext];
    return true;
}

getDefaultEngine = exports.getDefaultEngine = function()
{
    engine = Object.keys(_renderer.engine)[0];
    if (!engine) throw new ReferenceError("No default render engine available");
    return engine;
}

engines = exports.engines = function()
{
    return Object.keys(_renderer.engine);
}

// do we have a caching method?
renderCacheMethod = exports.renderCachedMethod = function(callbackName,engine) 
{ 
    if (!engine) engine = getDefaultEngine();
    if (!callbackName) throw new SyntaxError("No callback specified");
    if (_renderer.engine[engine][callbackName] == undefined)
        throw new ReferenceError("Invalid callback");
    _renderer.engine[engine].renderCachedMethod = _renderer.engine[engine][callbackName];
    return true;
};

// standard rendering method
renderMethod = exports.renderMethod = function(callbackName,engine) 
{ 
    if (!engine) engine = getDefaultEngine();
    if (!callbackName) throw new SyntaxError("No callback specified");
    if (_renderer.engine[engine][callbackName] == undefined)
        throw new ReferenceError("Invalid callback");
    _renderer.engine[engine].renderMethod = _renderer.engine[engine][callbackName];
    return true;
}

// standard compiling method
compileMethod = exports.compileMethod = function(callbackName,engine) 
{ 
    if (!engine) engine = getDefaultEngine();
    if (!callbackName) throw new SyntaxError("No callback specified");
    if (_renderer.engine[engine][callbackName] == undefined)
        throw new ReferenceError("Invalid callback");
    _renderer.engine[engine].compileMethod = _renderer.engine[engine][callbackName];
    return true;
}

// getter/setter for requiring caching - this is a global setting
cache = exports.cache = function (caching)
{
    if (caching == undefined) { return _renderer.caching };
    _renderer.caching = caching;
}

path = exports.path = function(path)
{
  if (!path) { return _renderer.path }
  _renderer.path = path;
}

//NOT IMPLEMENTED
memcache = exports.memcache = function() {}

//flushes templates from either a specified engine or all engines otherwise.
// @todo: once memcache is added, make sure the templates are removed from
//          there as well.
flush = exports.flush = function(engine) 
{
    if(engine) 
    {
        _renderer.engine[engine].templates = {};
    } else
    {
        Object.keys(_renderer.engine).forEach(function(engine)
            {
                _renderer.engine[engine].templates = {};
            });
    }
}

registerTemplate = exports.registerTemplate = function(name,markup,engine)
{
    //nothing? Fall back to the default
    if (!engine) engine = getDefaultEngine();
        
    //still nothing? Danger, Will Robinson
    if (!engine) throw new Error("No render engine found");
   
    if ((!name) || (name == '')) throw new SyntaxError("No template name provided");

    //initialize template property if necessary
    if (!_renderer.engine[engine].templates) _renderer.engine[engine].templates = {};
    if (!_renderer.engine[engine].templates[name]) _renderer.engine[engine].templates[name] = {};

    _renderer.engine[engine].templates[name].markup = markup;
    return true;
}

//Above, from a file. Calls above.
registerTemplateFile = exports.registerTemplateFile = function(name,filename,engine)
{
    if ((!name) || (!filename)) throw new Error("Can not register template");
    
    if (!engine)
    {
        //guess the engine
        _e = filename.split('.').pop();
        if ((_e) && (_renderer.engine[_e] != undefined)) engine = _e;

        //nothing? Fall back to the default
        if (!engine) engine = getDefaultEngine();
    }

    if (!fs.statSync(path() + '/' + filename).isFile()) throw new Error("No such file: " + filename);

    //initialize template property if necessary
    if (!_renderer.engine[engine].templates[name]) _renderer.engine[engine].templates[name] = {};

    //set extra property
    _renderer.engine[engine].templates[name].filename = filename;

    //do the rest
    _f = fs.readFileSync(path() + '/' + filename);

    registerTemplate(name,_f.toString(),engine);
    return true;
}

//alias
exports.template = registerTemplate;

isTemplate = exports.isTemplate = function(name,engine) {

    if (!name) throw new SyntaxError("No name provided")
    if (!engine) engine = getDefaultEngine();

    if (!_renderer.engine[engine].templates[name]) return false;
    return true;
}

//Returns an array of the names of the available templates
templates = exports.templates = function(engine)
{
    if (!engine) engine = getDefaultEngine();
    return Object.keys(_renderer.engine[engine].templates);
}

//item: name or template data; 
render = exports.render = function(item,data,engine)
{
    if ((_renderer == {}) || (!_renderer.engine)) throw new Error("No renderer available");
    if (!engine) engine = getDefaultEngine();

    if (!item) throw new Error("invalid render call: no template or markup");

    // do we have a renderer callback set?
    if (_renderer.engine[engine].renderMethod == undefined)
    {
        //if not, try and guess.
        if (_renderer.engine[engine].render != undefined)
        {
            //this looks silly to read
            _renderer.engine[engine].renderMethod = _renderer.engine[engine].render;
        } else
        {
            throw new Error("No method available to render: " + item);
        }
    }

    if ((!_renderer.engine[engine].templates) || (!_renderer.engine[engine].templates[item])) {
        markup = item;
    } else
    {
        markup = _renderer.engine[engine].templates[item].markup;
    }


    return _renderer.engine[engine].renderMethod(markup,data);

}

//Not yet implemented
compile = exports.compile = function(item,data,engine) {}
