var vows = require('vows');
    assert = require('assert');

// required for tests
var ejs = require('ejs');
var jqtpl = require('jqtpl');

var groundcrew = require('../lib/groundcrew');

vows.describe("groundcrew operations tests ").addBatch({
        "Set up: ": {
            'groundcrew operational': {
                topic: function() { return groundcrew },
                'Is a groundcrew': function(topic) { 
                        assert.isTrue(topic.new()); 
                    }
            },
            'caching is:': {
                topic: function() { return groundcrew },
                'off by default': function(topic) {
                        topic.new(ejs,'ejs'); 
                        assert.isFalse(topic.cache()); 
                    },
                'explicitly turned on at start': function(topic) {
                        topic.new(ejs,'ejs',true);
                        assert.isTrue(topic.cache());
                    },
                'explicitly turned off at start': function(topic) {
                        topic.new(ejs,'ejs',false);
                        assert.isFalse(topic.cache());
                    },
                'turned on': function(topic) { 
                        topic.new(ejs,'ejs'); 
                        topic.cache(true); 
                        assert.isTrue(topic.cache()); 
                    },
                'turned off': function(topic) { 
                        topic.new(ejs,'ejs'); 
                        topic.cache(true); 
                        topic.cache(false); 
                        assert.isFalse(topic.cache()); 
                    }
            },
            'engine registration:': {
                topic: function() { return groundcrew },
                'success': function(topic) {
                        assert.isTrue(topic.registerEngine(ejs,'ejs'));
                    },
                'failure': function(topic) {
                        assert.throws(function() { topic.registerEngine() },Error);
                    },
                'two engines registered': function(topic) {
                        topic.new();
                        topic.registerEngine(ejs,'ejs');
                        topic.registerEngine(jqtpl,'html');
                        assert.equal(topic.engines().length,2);
                    },
                'default engine removal failure': function(topic) {
                        topic.new(ejs,'ejs');
                        assert.throws(function() { topic.unregisterEngine('ejs') },Error);
                    },
                'unregistered engine removal failure': function(topic) {
                        assert.throws(function() { topic.unregisterEngine('ejs') },Error);
                    },
                'unregister engine success': function(topic) {
                        topic.new();
                        topic.registerEngine(ejs,'ejs');
                        topic.registerEngine(jqtpl,'html');
                        assert.isTrue(topic.unregisterEngine('ejs'));
                    }

            }
        }
}).addBatch({
    "Execution: ": {
        'set render method checks': {
                topic: function() { return groundcrew },
                'set compile method success': function(topic) {
                        topic.new(ejs,'ejs'); 
                        assert.isTrue(topic.compileMethod("compile"));
                    },
                'set compile method failure': function(topic) {
                        topic.new(ejs,'ejs'); 
                        assert.throws(function() { topic.compileMethod(); },Error);
                    },
                'set render method success': function(topic) {
                        topic.new(ejs,'ejs'); 
                        assert.isTrue(topic.renderMethod("render"));
                    },
                'set render method failure': function(topic) {
                        topic.new(ejs,'ejs'); 
                        assert.throws(function() { topic.renderMethod(); },Error);
                    },
                'set (cached)render method success': function(topic) {
                        topic.new(ejs,'ejs'); 
                        // ejs doesn't have a caching render method
                        // this is more just to make sure we can set it
                        assert.isTrue(topic.renderCachedMethod("render"));
                    },
                'set (cached)render method failure': function(topic) {
                        topic.new(ejs,'ejs'); 
                        assert.throws(function() { topic.renderCacheMethod(); },Error);
                    }
        },
        'templating': {
            "registration": {
                topic: function() 
                    { 
                        groundcrew.new();
                        groundcrew.registerEngine(ejs,'ejs');
                        groundcrew.registerEngine(jqtpl,'html');
                        groundcrew.path(__dirname + '/templates');
                        return groundcrew;
                    },
                'markup with explicit engine': function(topic) 
                    {
                        assert.isTrue(topic.registerTemplate('test_jqtpl','{$title}','html'));
                    },
                'markup with implicit engine': function(topic) 
                    {
                        assert.isTrue(topic.registerTemplate('test_ejs','<%= title %>'));
                    },
                'file with explicit engine': function(topic) 
                    {
                        assert.isTrue(topic.registerTemplateFile('filetest_jqtpl','test.html','html'));
                    },
                'file with implicit engine': function(topic) 
                    {
                        assert.isTrue(topic.registerTemplateFile('filetest_ejs','test.ejs'));
                    },
                 'No name': function(topic) 
                    {
                        assert.throws(function() { topic.registerTemplate() }, Error);
                    },
                 'Bad file name': function(topic)
                    {
                        assert.throws(function() { topic.registerTemplateFile('bad.ejs') }, Error);
                    }

            }

        }
    }
}).addBatch({
    "Rendering: ": {
        "implicit": {
            topic: function() 
                {
                    groundcrew.new(ejs,'ejs');
                    groundcrew.path(__dirname + '/templates');
                    return groundcrew;
                },
            "from markup": function(topic) 
                {
                    _r = topic.render('<%= title %>',{locals: {title: "test"} });
                    assert.equal(_r,'test');
                },
            "from registered markup": function(topic)
                {
                    topic.registerTemplate('test_ejs','<%= title %>');
                    _r = topic.render('test_ejs',{locals: {title: "test"} });
                    assert.equal(_r,'test');
                },
            "from file": function(topic)
                {
                    topic.registerTemplateFile('filetest_ejs','test.ejs');
                    _r = topic.render('filetest_ejs',{locals: {title: "test"}});
                    assert.equal(_r.trim(),'test');
                }
        }
    }
}).addBatch({
    "Rendering: ": {
        "explicit": {
            topic: function() 
                {
                    groundcrew.new(jqtpl,'html');
                    groundcrew.path(__dirname + '/templates');
                    groundcrew.renderMethod('tmpl','html');
                    return groundcrew;
                },
            "from markup": function(topic) 
                {
                    _r = topic.render('${title}', {title: "test"} , 'html' );
                    assert.equal(_r,'test');
                },
            "from registered markup": function(topic)
                {
                    topic.registerTemplate('test_jqtpl','${title}','html');
                    _r = topic.render('test_jqtpl', {title: "test"}, 'html' );
                    assert.equal(_r,'test');
                },
            "from file": function(topic)
                {
                    topic.registerTemplateFile('filetest_jqtpl','test.html');
                    _r = topic.render('filetest_jqtpl',{title: "test"});
                    assert.equal(_r.trim(),'test');
                }
        }
    }
}).export(module);
